function [u,residual,nbit,d1,d2] = SB_integration(p,q,mask,lambda,alpha,u0,tol,maxit,nGS,display)
% Split-Bregman integration of normals
%
%	Reference : Edge-Preserving Integration of a Normal Field: Weighted Least-squares, TV and L1 Approaches, Yvain Queau and Jean-Denis Durou, SSVM 2015
%
%	Minimises : 
%	F(u) = \iint_{mask} w(x,y) ||\nabla u(x,y) - [p(x,y),q(x,y)]||_1 + \lambda (u(x)-u0(x))^2 dx dy
%
%	Required : 
%	p : estimate derivatives of u in the x-direction (towards bottom)
%	q : estimate derivatives of u in the y-direction (towards right)
%	
%	Optional : 
%	mask : binary mask (should at least be connected)	
%	lambda : fidelity weight (default is 1e-4)
%	alpha : split-Bregman parameter (default is 1e-1)
%	u0 : quadratic prior (default is simchony(p,q), filling p and q with zero outside the mask)
%	tol : mean quadratic residual stopping criterion (default is 5e-4)
%	maxit : maximum number of iterations (default is 500)
%	nGS : number of Gauss-Seidel relaxations per iterations (default is 5)
%	display : binary variable for displaying the evolution of the depth map, the residual, and the norms of the Bregman variables (much slower, but useful for debug)
%
%
%	Author : Yvain Queau - University of Toulouse - IRIT, UMR CNRS 5505
%	yvain.queau@enseeiht.fr
%	http://ubee.enseeiht.fr/photometricstereo//
	
	if(nargin<2)
		disp('estimated gradient (p,q) must be provided')
		return;
	end
	p(find(isnan(p)))=0;
	q(find(isnan(q)))=0;
	if (~exist('mask','var')|isempty(mask)) mask=ones(size(p));end
	indices_mask = find(mask>0);
	indices_nmask = find(mask==0);
	if (~exist('lambda','var')|isempty(lambda)) lambda=1e-4;end
	if (~exist('alpha','var')|isempty(alpha)) alpha=1e-1;end
	p(indices_nmask)=0;
	q(indices_nmask)=0;
	if (~exist('u0','var')|isempty(u0)) u0=simchony(p,q);end
	if (~exist('tol','var')|isempty(tol)) tol=5e-4;end	
	if (~exist('maxit','var')|isempty(maxit)) maxit=500;end	
	if (~exist('nGS','var')|isempty(nGS)) nGS=5;end
	if (~exist('display','var')|isempty(display)) display=0;end
	
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	% Calculate some usefuk masks : 								   %	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	Omega = zeros(size(mask,1),size(mask,2),4);
	Omega_padded = padarray(mask,[1 1],0);
	Omega(:,:,1) = Omega_padded(3:end,2:end-1).*mask;	% In mask, neigbor "under" in mask 
	indices_u = find(Omega(:,:,1)>0);
	Omega(:,:,2) = Omega_padded(1:end-2,2:end-1).*mask; % In mask, neigbor "over" in mask 
	Omega(:,:,3) = Omega_padded(2:end-1,3:end).*mask;	% In mask, neigbor "right" in mask 
	indices_r = find(Omega(:,:,3)>0);
	Omega(:,:,4) = Omega_padded(2:end-1,1:end-2).*mask; % In mask, neigbor "left" in mask 
	clear Omega_padded 	

	% Compute the system Ax = B
	mapping_matrix = zeros(size(p));
	mapping_matrix(indices_mask)=1:length(indices_mask); 
	pbar = 0.5*(p+p([2:end end],:));
	qbar = 0.5*(q+q(:,[2:end end]));
	deux_lambda_sur_alpha = 2*lambda/alpha;
	[A,B] = calculate_system(pbar,qbar,mask,Omega,deux_lambda_sur_alpha,u0,ones(size(mask)),mapping_matrix);
	clear Omega mapping_matrix
	
	% Init solution
	u = u0;u(indices_nmask)=NaN;
	d1 = zeros([size(mask)]);
	d2 = zeros([size(mask)]);
	b1 = zeros([size(mask)]);
	b2 = zeros([size(mask)]);
		
	% Init GS
    x = u(indices_mask);
    r=B-A*x;
    trilA = tril(A);
    nabla_u_minus_g_plus_b = zeros([size(mask) 2]);
    norm_u_minus_g_plus_b = zeros(size(mask));
    un_sur_alpha = 1/alpha;
          
    if(display)
		h1 = figure();
		hdisp=imagesc(fliplr(u));
		colormap jet
		axis equal
		axis off
		axis ij
		colorbar
		title('Depth map')
		drawnow
		
		h1 = figure();
		hdisp2=imagesc(fliplr(b1.^2+b2.^2));
		colormap jet
		axis equal
		axis off
		axis ij
		colorbar
		title('$||b||^2$','Interpreter','Latex')
		drawnow
		
		h3 = figure();
		hdisp3=imagesc(fliplr(d1.^2+d2.^2));
		colormap jet
		axis equal
		axis off
		axis ij
		colorbar
		title('$||d||^2$','Interpreter','Latex')
		drawnow
		
		h4 = figure();
		hdisp4=imagesc(fliplr(u-u));
		colormap jet
		axis equal
		axis off
		axis ij
		colorbar
		title('$u^{k+1}-u^k$','Interpreter','Latex')
		drawnow
	end
          
    for iBreg = 1:20 % Do at least 20 iterations      
		u_before = u;
		for iGS=1:nGS
			%%% Gauss-Seidel update of u			
			% Ubdate rhs of the system
			pi = p+d1-b1;
			qi = q+d2-b2;
			pibar = 0.5*(pi+pi([2:end end],:));
			qibar = 0.5*(qi+qi(:,[2:end end]));
			pibar(indices_nmask) = 0;
			qibar(indices_nmask) = 0;
			Bnew = -deux_lambda_sur_alpha*u0+qibar-[zeros(size(mask,1),1) qibar(:,1:end-1)]+pibar-[zeros(1,size(mask,2)); pibar(1:end-1,:)];
			B(:) = Bnew(indices_mask);
			
			% EffectiveGauss-Seidel update
			x=x+trilA\r; 
			r=B-A*x;     
			u(indices_mask)=x;
		end
			
		%%% Shrinkage operations			
		u_x = u([2:end end],:)-u;
		u_y = u(:,[2:end end])-u;
		nabla_u_minus_g_plus_b1 = u_x - pbar+b1;
		nabla_u_minus_g_plus_b2 = u_y - qbar+b2;
		abs_nabla_u_minus_g_plus_b1 = abs(nabla_u_minus_g_plus_b1);
		abs_nabla_u_minus_g_plus_b2 = abs(nabla_u_minus_g_plus_b2);
		num1 = max(0,abs_nabla_u_minus_g_plus_b1-un_sur_alpha);
		num2 = max(0,abs_nabla_u_minus_g_plus_b2-un_sur_alpha);
		denom1 = max(eps,abs_nabla_u_minus_g_plus_b1);
		denom2 = max(eps,abs_nabla_u_minus_g_plus_b2);
		d1(indices_u) = num1(indices_u).*(nabla_u_minus_g_plus_b1(indices_u))./denom1(indices_u);
		d2(indices_r) = num2(indices_r).*(nabla_u_minus_g_plus_b2(indices_r))./denom2(indices_r);
		
		%%% Bregman update
		b1(indices_u) = b1(indices_u)+u_x(indices_u)-pbar(indices_u)-d1(indices_u);
		b2(indices_r) = b2(indices_r)+u_y(indices_r)-qbar(indices_r)-d2(indices_r);
    end
    residual = norm(u_before(indices_mask)-u(indices_mask))/norm(u(indices_mask));
    
    
    for iBreg = 21:maxit % Do the remaining iterations
		u_before = u;
		for iGS=1:nGS
			%%% Gauss-Seidel update of u			
			% Ubdate rhs of the system
			pi = p+d1-b1;
			qi = q+d2-b2;
			pibar = 0.5*(pi+pi([2:end end],:));
			qibar = 0.5*(qi+qi(:,[2:end end]));
			pibar(indices_nmask) = 0;
			qibar(indices_nmask) = 0;
			Bnew = -deux_lambda_sur_alpha*u0+qibar-[zeros(size(mask,1),1) qibar(:,1:end-1)]+pibar-[zeros(1,size(mask,2)); pibar(1:end-1,:)];
			B(:) = Bnew(indices_mask);
			
			% EffectiveGauss-Seidel update
			x=x+trilA\r; 
			r=B-A*x;     
			u(indices_mask)=x;
		end
			
		%%% Shrinkage operations			
		u_x = u([2:end end],:)-u;
		u_y = u(:,[2:end end])-u;
		nabla_u_minus_g_plus_b1 = u_x - pbar+b1;
		nabla_u_minus_g_plus_b2 = u_y - qbar+b2;
		abs_nabla_u_minus_g_plus_b1 = abs(nabla_u_minus_g_plus_b1);
		abs_nabla_u_minus_g_plus_b2 = abs(nabla_u_minus_g_plus_b2);
		num1 = max(0,abs_nabla_u_minus_g_plus_b1-un_sur_alpha);
		num2 = max(0,abs_nabla_u_minus_g_plus_b2-un_sur_alpha);
		denom1 = max(eps,abs_nabla_u_minus_g_plus_b1);
		denom2 = max(eps,abs_nabla_u_minus_g_plus_b2);
		d1(indices_u) = num1(indices_u).*(nabla_u_minus_g_plus_b1(indices_u))./denom1(indices_u);
		d2(indices_r) = num2(indices_r).*(nabla_u_minus_g_plus_b2(indices_r))./denom2(indices_r);
		
		%%% Bregman update
		b1(indices_u) = b1(indices_u)+u_x(indices_u)-pbar(indices_u)-d1(indices_u);
		b2(indices_r) = b2(indices_r)+u_y(indices_r)-qbar(indices_r)-d2(indices_r);

		% Convergence ?
		residual = norm(u_before(indices_mask)-u(indices_mask))/norm(u(indices_mask));
		if(display)
			disp(sprintf('Iteration %d : residual = %.08f',iBreg,residual));
			set(hdisp,'CData',fliplr(u))
			drawnow
			
			set(hdisp2,'CData',fliplr(b1.^2+b2.^2))
			drawnow
			
			set(hdisp3,'CData',fliplr(d1.^2+d2.^2))
			drawnow
			
			set(hdisp4,'CData',fliplr(u-u_before))
			drawnow
		end
		if(residual<tol)
			break;
		end
    end
    
	nbit = iBreg;
	
	%%% That's all folks ! %%%
	
end



function [A,B] = calculate_system(pbar,qbar,mask,Omega,lambda,u0,weights,mapping_matrix)

	I = [];
	J = [];
	K = [];
	indices_mask = find(mask>0);
	B = zeros(length(indices_mask),1);

	% All px in mask
	[X,Y]=find(mask>0);
	indices_center=sub2ind(size(mask),X,Y);
	I_centre = mapping_matrix(indices_center);
	A_centre = -lambda *ones(length(I_centre),1);	
	I=[I;I_centre]; 
	J=[J;I_centre]; 
	K=[K;A_centre]; 
	B(I_centre)=-lambda*u0(indices_mask);	

	% In mask, right neighbor in mask
	set = Omega(:,:,3);
	[X,Y]=find(set>0);
	indices_centre=sub2ind(size(mask),X,Y);
	I_centre = mapping_matrix(indices_centre);
	indices_voisins=sub2ind(size(mask),X,Y+1);
	I_voisins = mapping_matrix(indices_voisins);
	A_centre = -(weights(indices_centre));
	A_voisin =weights(indices_centre);
	
	K=[K;A_centre(:);A_voisin(:)]; 
	I=[I;I_centre(:);I_centre(:)]; 
	J=[J;I_centre(:);I_voisins(:)]; 
	B(I_centre) = B(I_centre)+weights(indices_centre).*qbar(indices_centre);
	
	
	% In mask, left neighbor in mask
	set = Omega(:,:,4);
	[X,Y]=find(set>0);
	indices_centre=sub2ind(size(mask),X,Y);
	I_centre = mapping_matrix(indices_centre);
	indices_voisins=sub2ind(size(mask),X,Y-1);
	I_voisins = mapping_matrix(indices_voisins);
	A_centre = -(weights(indices_voisins));
	A_voisin =weights(indices_voisins);
	K=[K;A_centre(:);A_voisin(:)]; 
	I=[I;I_centre(:);I_centre(:)]; 
	J=[J;I_centre(:);I_voisins(:)]; 
	B(I_centre) = B(I_centre)-weights(indices_voisins).*qbar(indices_voisins);
	
	% In mask, top neighbor in mask
	set = Omega(:,:,2);
	[X,Y]=find(set>0);
	indices_centre=sub2ind(size(mask),X,Y);
	I_centre = mapping_matrix(indices_centre);
	indices_voisins=sub2ind(size(mask),X-1,Y);
	I_voisins = mapping_matrix(indices_voisins);
	A_centre = -(weights(indices_voisins));
	A_voisin =weights(indices_voisins);
	K=[K;A_centre(:);A_voisin(:)]; % Which values ?
	I=[I;I_centre(:);I_centre(:)]; % Which values ?
	J=[J;I_centre(:);I_voisins(:)]; % Which values ?
	B(I_centre) = B(I_centre)-weights(indices_voisins).*pbar(indices_voisins);	
		
	% In mask, bottom neighbor in mask
	set = Omega(:,:,1);
	[X,Y]=find(set>0);
	indices_centre=sub2ind(size(mask),X,Y);
	I_centre = mapping_matrix(indices_centre);
	indices_voisins=sub2ind(size(mask),X+1,Y);
	I_voisins = mapping_matrix(indices_voisins);
	A_centre = -(weights(indices_centre));
	A_voisin =weights(indices_centre);
	K=[K;A_centre(:);A_voisin(:)]; 
	I=[I;I_centre(:);I_centre(:)]; 
	J=[J;I_centre(:);I_voisins(:)]; 
	B(I_centre) = B(I_centre)+weights(indices_centre).*pbar(indices_centre);	
	
	% Construction de A : 
	A = sparse(I,J,K);	
	
end


function [U,rmse] = simchony(p,q,gt)
% An implementation of the method from Simchony et Al for integration 
% of a normal field with natural boundary condition
% Ref : Direct Analytical Methods for Solving Poisson Equation in 
% Computer Vision problems - PAMI 1990
%
% example :
% p = ones(100,100); 
% q = ones(100,100); 
% u = simchony(p,q); 
%
% This performs the least square solution to \nabla u = [p,q], i.e. :
% min \int_\Omega \| \nablua U - [p,q] \|^2
% where \Omega is square and the natural Neumann boundary condition 
% \mu \cdotp (\nabla u -[p,q]) = 0 is used (2nd order derivatives 
% are neglected), where \mu is the outer 
% normal to \Omega. 
% This boundary condition is the one given by the 
% calculus of variations.
%
% % Axis : O->y
%        |
%        x
%
% The problem comes to solve a linear system Ax=b, where A is a bloc 
% Toplitz matrix. Fast solution is provided by Discrete Cosine Transform
%
% Implementation : Yvain Queau
% Universite de Toulouse, IRIT, UMR CNRS 5505
% yvain.queau@enseeiht.fr
% See other codes at http://ubee.enseeiht.fr/photometricstereo/

%~ p = 0.5*(p([2:end end],:)+p);
%~ q = 0.5*(q(:,[2:end end])+q);

% Compute div(p,q)
px = 0.5*(p([2:end end],:)-p([1 1:end-1],:));
qy = 0.5*(q(:,[2:end end])-q(:,[1 1:end-1]));

% Div(p,q) + Boundary Condition
f = px+qy;
f(1,2:end-1) = 0.5*(p(1,2:end-1)+p(2,2:end-1));
f(end,2:end-1) = 0.5*(-p(end,2:end-1)-p(end-1,2:end-1));
f(2:end-1,1) = 0.5*(q(2:end-1,1)+q(2:end-1,2));
f(2:end-1,end) = 0.5*(-q(2:end-1,end)-q(2:end-1,end-1));

f(1,1)=0.5*(p(1,1)+p(2,1)+q(1,1)+q(1,2));
f(end,1)=0.5*(-p(end,1)-p(end-1,1)+q(end,1)+q(end,2));
f(1,end)=0.5*(p(1,end)+p(2,end)-q(1,end)-q(1,end-1));
f(end,end)=0.5*(-p(end,end)-p(end-1,end)-q(end,end)-q(end,end-1));

% Sine transform of f
fsin=dct2(f);

% Denominator
[x,y] = meshgrid(0:size(p,2)-1,0:size(p,1)-1);
denom = (2*cos(pi*x/(size(p,2)))-2) + (2*cos(pi*y/(size(p,1))) - 2);
Z = fsin./(denom);
Z(1,1)=0.5*Z(1,2)+0.5*Z(2,1); %Or whatever...

% Inverse Sine transform :
U=idct2(Z);


	if(nargin>2)% Ground truth available
		moyenne_ecarts=mean(U(:)-gt(:));
		U=U-moyenne_ecarts;
		npix=size(p,1)*size(p,2);
		rmse=sqrt((sum((U(:)-gt(:)).^2))/npix);
	else
		U=U-min(U(:));
	end

end



