% Demo script for split-Bregman integration of normals 

% See SB_integration.m for details

% Reference : Edge-Preserving Integration of a Normal Field: Weighted Least-squares, TV and L1 Approaches, Yvain Queau and Jean-Denis Durou, SSVM 2015

clear 
close all

% Load a gradient field estimated by photometric stereo
load Beethoven; % Credits : http://vision.seas.harvard.edu/qsfs/Data.html
% This dataset contains p,q,mask, and the photometric stereo data

% Show the input gradient
figure()
hold on
subplot(1,2,1)
imagesc(p)
axis equal
axis off
title('$p$','Interpreter','Latex')
subplot(1,2,2)
imagesc(q)
axis equal
axis off
title('$q$','Interpreter','Latex')
hold off
st = suptitle('Input gradient field $g=[p,q]$');
set(st,'Interpreter','Latex')
drawnow;pause(0.1);

% Least squares solution by DCT, filling out values with zeros
p(find(mask==0)) = 0;
q(find(mask==0)) = 0;
u_DCT = simchony(p,q);

h = figure();
surfl(u_DCT,[0 90]);
shading interp
colormap(gray);
axis equal
axis ij
axis off
view(-45,30);
camlight(90,-60)
camlight(90,-60)
title('DCT reconstruction')
drawnow;pause(0.1);

% Split-Bregman solution, filling out values with zeros
p(find(mask==0)) = 0;
q(find(mask==0)) = 0;
u_SB = SB_integration(p,q);

h = figure();
surfl(u_SB,[0 90]);
shading interp
colormap(gray);
axis equal
axis ij
axis off
view(-45,30);
camlight(90,-60)
camlight(90,-60)
title('Split-Bregman reconstruction (full mask)')
drawnow;pause(0.1);

% Split-Bregman solution, using masked pixels
p(find(mask==0)) = 0;
q(find(mask==0)) = 0;
u_SB_masked = SB_integration(p,q,mask);

h = figure();
surfl(u_SB_masked,[0 90]);
shading interp
colormap(gray);
axis equal
axis ij
axis off
view(-45,30);
camlight(90,-60)
camlight(90,-60)
title('Masked Split-Bregman reconstruction')
drawnow;pause(0.1);

% Another test, with full set of parameters, to demonstrate what we can do with flat initialisation, lower stopping criterion, etc.
mask = ones(size(p)); % Mask = full set of pixels
lambda = 1e-9; % Very low influence of the quadratic prior
alpha = 1e-1; % alpha >> lambda : only diffusion matters
u0 = zeros(size(p)); % Flat quadratic prior
tol = 2e-3; % Stopping criterion
maxit = 500; % Max number of iterations
nGS = 2; % Number of Gauss-Seidel relaxations per iterations
display = 1;
u_SB_flat_prior = SB_integration(p,q,mask,lambda,alpha,u0,tol,maxit,nGS,display);

h = figure();
surfl(u_SB_flat_prior,[0 90]);
shading interp
colormap(gray);
axis equal
axis ij
axis off
view(-45,30);
camlight(90,-60)
camlight(90,-60)
title('Split-Bregman reconstruction (flat prior)')
drawnow;pause(0.1);

